package cornell.ece4760.timesystem;

import android.app.TabActivity;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Window;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.TabHost.TabSpec;

/**
 * This is the main activity that is responsible for
 * bluetooth connectivity, maintaining connection across
 * tabs, and tab set up and control.
 * Partial bluetooth code referenced from Google's example
 * 
 * @author Andy Liang
 *
 */

public class TimeSystem extends TabActivity {
	// Debugging
	private static final String TAG = "TimeSystem";
	private static final boolean D = true;
	// Layout Views
	private static TextView mTitle;
	private TabHost mTabHost;

	// Message types sent from the BluetoothService Handler
	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int MESSAGE_WRITE = 3;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;

	// Key names received from the BluetoothService Handler
	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";

	// Intent request codes
	private static final int REQUEST_CONNECT_DEVICE = 1;
	private static final int REQUEST_ENABLE_BT = 2;

	// Name of the connected device
	private String mConnectedDeviceName = null;
	// Local Bluetooth adapter
	public static BluetoothAdapter mBluetoothAdapter = null;
	// Member object for the chat services
	public static BluetoothService mService = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		if(D) Log.e(TAG, "+++ ON CREATE +++");

		// Set up the window layout
		requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
		setContentView(R.layout.main);
		getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.custom_title);

		// Set up the custom title
		mTitle = (TextView) findViewById(R.id.title_left_text);
		mTitle.setText(R.string.app_name);
		mTitle = (TextView) findViewById(R.id.title_right_text);

		// extract TabHost object
		mTabHost= (TabHost) findViewById(android.R.id.tabhost);
		Intent intent= new Intent().setClass(this,Bluetooth.class);

		// Get local Bluetooth adapter
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		// If the adapter is null, then Bluetooth is not supported
		if (mBluetoothAdapter == null) {
			Toast.makeText(this, "Bluetooth is not available on your device", Toast.LENGTH_LONG).show();
			finish();
			return;
		}

		if (mService == null) mService = new BluetoothService(this, mHandler);

		// First tab
		View tabview = createTabView(mTabHost.getContext(), "Remote Control");
		TabSpec setContent = mTabHost.newTabSpec("Remote Control").setIndicator(tabview).setContent(intent);
		mTabHost.addTab(setContent);

		// Second tab
		intent= new Intent().setClass(this, Routine.class);
		tabview = createTabView(mTabHost.getContext(), "Routine Config");
		setContent = mTabHost.newTabSpec("Routine Config").setIndicator(tabview).setContent(intent);
		mTabHost.addTab(setContent);

		mTabHost.setCurrentTab(0);
	}

	@Override
	public void onStart() {
		super.onStart();
		if(D) Log.e(TAG, "++ ON START ++");
		if (!mBluetoothAdapter.isEnabled()) {
			Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
			// Otherwise, setup the chat session
		} else {
			if (mService == null) mService = new BluetoothService(this, mHandler);
		}
	}

	@Override
	public synchronized void onResume() {
		super.onResume();
		if(D) Log.e(TAG, "+ ON RESUME +");

		// Performing this check in onResume() covers the case in which BT was
		// not enabled during onStart(), so we were paused to enable it...
		// onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
		if (mService != null) {
			// Only if the state is STATE_NONE, do we know that we haven't started already
			if (mService.getState() == BluetoothService.STATE_NONE) {
				mService.start();
			}
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		// Stop the Bluetooth chat services
		if (mService != null) mService.stop();
		if(D) Log.e(TAG, "--- ON DESTROY ---");

	}

	private static View createTabView(final Context context, final String text) {
		View view= LayoutInflater.from(context).inflate(R.layout.tabs_bg, null);
		TextView tv= (TextView) view.findViewById(R.id.tabsText);
		tv.setText(text);
		return view;
	}

	public static TextView getMTitle() {
		return mTitle;
	}

	// The Handler that gets information back from the BluetoothService
	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MESSAGE_STATE_CHANGE:
				if(D) Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
				switch (msg.arg1) {
				case BluetoothService.STATE_CONNECTED:
					mTitle.setText(R.string.title_connected_to);
					mTitle.append(mConnectedDeviceName);
					break;
				case BluetoothService.STATE_CONNECTING:
					mTitle.setText(R.string.title_connecting);
					break;
				case BluetoothService.STATE_LISTEN:
				case BluetoothService.STATE_NONE:
					mTitle.setText(R.string.title_not_connected);
					break;
				}
				break;
			case MESSAGE_WRITE:
				break;
			case MESSAGE_READ:
				break;
			case MESSAGE_DEVICE_NAME:
				// save the connected device's name
				mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
				Toast.makeText(getApplicationContext(), "Connected to "
						+ mConnectedDeviceName, Toast.LENGTH_SHORT).show();
				break;
			case MESSAGE_TOAST:
				Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
						Toast.LENGTH_SHORT).show();
				break;
			}
		}
	};


}
